%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% File name: Init.m
%
% Description: Initialization algorithm using Kalman theory, 
%              to find initial parameters for the RPEM.
%
% Inputs: Input signals, dimensions and output signals. 
%
% Outputs: Estimated parameters.
%
% Author and copyright: Linda Brus
%                       Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       Uppsala
%                       Sweden
%
% Revision log: Created 2005-05-10.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Dimensions
% 

%  PolynomialOrders=zeros(nx+nu,1);
%
%  Example:
%
%  polynomialOrders=[3 1 3]';
%  notUsedIndices=[
%   1 1 1;
%   2 1 1;
%   3 1 1];
%
%  Note: First index corresponds to x_1 (slowest variation), the 
%        following to x_2 and so on.
%

%
% Generate indices to sum over
%

if ~exist('theta_0_new')
   'Input polynomialOrders and notUsedIndices'
   keyboard;
   allIndices=GenerateIndices(polynomialOrders,notUsedIndices)
   dimTheta=size(allIndices,1);
   'Input theta_0'
   keyboard;
else
   allIndices=GenerateIndices(polynomialOrders,notUsedIndices);
   dimTheta=size(allIndices,1);
   theta_0=theta_0_new;
end

%
% Remaining initial values
%

R1=[R1_x*eye(nx) zeros(nx,dimTheta); zeros(dimTheta,nx) R1_theta*eye(dimTheta)];
P_0=scaleFactorP*eye(nx+dimTheta);


% Storage variables

x_m_save=zeros(nx,ceil(N/downSampling)+1);
y_m_save=zeros(ny,ceil(N/downSampling)+1);
perr_save=zeros(ny,ceil(N/downSampling));
theta_save=zeros(dimTheta,ceil(N/downSampling)+1);
SamplingInstances_DownSampled=zeros(1,ceil(N/downSampling)+1);

%
% Algorithmic loop
%

% Initiation at t=0...

y_m=y_m_0;
x_pred=[x_m_0; theta_0];       % Extended state vector
P_pred=P_0;
xi=[y_m_0; zeros(nx-1,1)];

x_m_save(:,1)=x_m_0;
y_m_save(:,1)=y_m_0;
theta_save(:,1)=theta_0;

percentReady=0;

for t=1:nx
  xi=differentiateY(y(t),xi,scalingTs*Ts);
end

for t=nx+1:N-1
   
   if round(t*100/(N-1))>percentReady
      percentReady=round(t*100/(N-1))
   end
 
   %
   % Estimate states from measured output
   %
   
   xi=differentiateY(y(t),xi,scalingTs*Ts);
   
   %
   % H-matrix for extended states
   %

   H=[dhdx_m(xi,u(:,t)) zeros(1,dimTheta)];
   
   %
   % Prediction error
   %
   
   perr=y(t)-H*x_pred;
   
   %
   % Kalman gain
   %
   
   K=P_pred*H'*inv(H*P_pred*H'+R2);
   
   %
   % State estimate
   %
   
   x_est=x_pred+K*(y(t)-H*x_pred);
   
   %
   % Covariance matrix estimate
   %
   
   P_est=P_pred-K*H*P_pred;
   
   %
   % System matrix computation 
   %
   
   F=eye(nx+dimTheta)+scalingTs*Ts*F_matrix(xi,u(:,t),x_pred(nx+(1:dimTheta),1),allIndices);
   
   %
   % State prediction
   % 
   
   x_pred=F*x_est;
   
   %
   % Linearized system matrix
   %
   
   F_tilde=F; 
   
   %
   % Covariance matrix prediction
   %
   
   P_pred=F_tilde*P_est*F_tilde'+R1;

   %
   % Model output computation
   %
   
   y_m=h_m(x_est,u(:,t));
   
   %
   % Check and save
   %
   
   if rem(t,downSampling)==0 % Time to save
      x_m_save(:,t/downSampling+1)=x_est(1:nx);
      y_m_save(:,t/downSampling+1)=y_m;
      perr_save(:,t/downSampling)=perr;
      theta_save(:,t/downSampling+1)=x_est(nx+1:nx+dimTheta);
      SamplingInstances_DownSampled(t/downSampling+1)=SamplingInstances(t);
   end
   
end

theta=x_est(nx+1:nx+dimTheta);
reInitiateTheta=theta;
reInitiateIndices=allIndices;

%
% Compute the re-scaled, original parameter vector
%

rescaledTheta=theta;
for i=1:dimTheta
   for j=1:nx
      thisSubIndex=allIndices(i,j);
      rescaledTheta(i,1)=rescaledTheta(i,1)*((1/scalingTs)^(thisSubIndex*(j-1)));
   end
end
rescaledTheta=rescaledTheta*(scalingTs)^nx;

%
% End of algorithmic loop
%

%
% Perform a stability check
%
   
SM=eye(nx)+scalingTs*Ts*dfdx_m(x_est(1:nx),u(:,t),theta,allIndices);
LinearizedPoles=eig(SM);
LinearizedPoleRadii=abs(LinearizedPoles)

[theta rescaledTheta]

